@@ -61,6 +61,13 @@ namespace orc {
 
     virtual void recordPosition(PositionRecorder* recorder) const override;
 
+    virtual void suppress() override;
+
+    /**
+     * Reset to initial state
+     */
+    void reset();
+
   protected:
     std::unique_ptr<BufferedOutputStream> outputStream;
     char* literals;
@@ -80,12 +87,7 @@ namespace orc {
                                 std::unique_ptr<BufferedOutputStream> output)
                                   : outputStream(std::move(output)) {
     literals = new char[MAX_LITERAL_SIZE];
-    numLiterals = 0;
-    tailRunLength = 0;
-    repeat = false;
-    bufferPosition = 0;
-    bufferLength = 0;
-    buffer = nullptr;
+    reset();
   }
 
   ByteRleEncoderImpl::~ByteRleEncoderImpl() {
@@ -203,6 +205,21 @@ namespace orc {
     recorder->add(static_cast<uint64_t>(numLiterals));
   }
 
+  void ByteRleEncoderImpl::reset() {
+    numLiterals = 0;
+    tailRunLength = 0;
+    repeat = false;
+    bufferPosition = 0;
+    bufferLength = 0;
+    buffer = nullptr;
+  }
+
+  void ByteRleEncoderImpl::suppress() {
+    // written data can be just ignored because they are only flushed in memory
+    outputStream->suppress();
+    reset();
+  }
+
   std::unique_ptr<ByteRleEncoder> createByteRleEncoder
                               (std::unique_ptr<BufferedOutputStream> output) {
     return std::unique_ptr<ByteRleEncoder>(new ByteRleEncoderImpl
@@ -324,7 +341,6 @@ namespace orc {
     inline void nextBuffer();
     inline signed char readByte();
     inline void readHeader();
-    inline void reset();
 
     std::unique_ptr<SeekableInputStream> inputStream;
     size_t remainingValues;
@@ -364,7 +380,9 @@ namespace orc {
     }
   }
 
-  void ByteRleDecoderImpl::reset() {
+  ByteRleDecoderImpl::ByteRleDecoderImpl(std::unique_ptr<SeekableInputStream>
+                                         input) {
+    inputStream = std::move(input);
     repeating = false;
     remainingValues = 0;
     value = 0;
@@ -372,12 +390,6 @@ namespace orc {
     bufferEnd = nullptr;
   }
 
-  ByteRleDecoderImpl::ByteRleDecoderImpl(
-                        std::unique_ptr<SeekableInputStream> input) {
-    inputStream = std::move(input);
-    reset();
-  }
-
   ByteRleDecoderImpl::~ByteRleDecoderImpl() {
     // PASS
   }
@@ -385,8 +397,10 @@ namespace orc {
   void ByteRleDecoderImpl::seek(PositionProvider& location) {
     // move the input stream
     inputStream->seek(location);
-    // reset the decoder status and lazily call readHeader()
-    reset();
+    // force a re-read from the stream
+    bufferEnd = bufferStart;
+    // read a new header
+    readHeader();
     // skip ahead the given number of records
     ByteRleDecoderImpl::skip(location.next());
   }
