@@ -411,22 +411,18 @@ public class ParserUtils {
     private final ColumnVector[] result;
     private int resultIdx = 0;
 
-    ColumnFinder(TypeDescription schema, ColumnVector[] columnVectors, int levels) {
+    ColumnFinder(TypeDescription schema, VectorizedRowBatch batch, int levels) {
       if (schema.getCategory() == TypeDescription.Category.STRUCT) {
-        top = columnVectors;
+        top = batch.cols;
         result = new ColumnVector[levels];
       } else {
         result = new ColumnVector[levels + 1];
-        current = columnVectors[0];
+        current = batch.cols[0];
         top = null;
         addResult(current);
       }
     }
 
-    ColumnFinder(TypeDescription schema, VectorizedRowBatch vectorizedRowBatch, int levels) {
-      this(schema, vectorizedRowBatch.cols, levels);
-    }
-
     private void addResult(ColumnVector vector) {
       result[resultIdx] = vector;
       resultIdx += 1;
@@ -463,11 +459,8 @@ public class ParserUtils {
                                                  boolean isCaseSensitive,
                                                  VectorizedRowBatch batch) {
     List<String> names = ParserUtils.splitName(source);
-    TypeDescription schemaToUse = removeAcid(schema);
-    ColumnVector[] columnVectors = SchemaEvolution.checkAcidSchema(schema)
-                  ? ((StructColumnVector) batch.cols[batch.cols.length - 1]).fields : batch.cols;
-    ColumnFinder result = new ColumnFinder(schemaToUse, columnVectors, names.size());
-    findColumn(schemaToUse, names, isCaseSensitive, result);
+    ColumnFinder result = new ColumnFinder(schema, batch, names.size());
+    findColumn(removeAcid(schema), names, isCaseSensitive, result);
     return result.result;
   }
 
@@ -587,7 +580,6 @@ public class ParserUtils {
     while (consumeChar(source, ',')) {
       params.add(parseName(source));
     }
-    return new MaskDescriptionImpl(maskName,
-        params.toArray(new String[params.size()]));
+    return new MaskDescriptionImpl(maskName, params.toArray(new String[0]));
   }
 }
